import 'package:built_collection/built_collection.dart';
import 'package:dart_twitter_api/twitter_api.dart';
import 'package:flutter/foundation.dart';
import 'package:harpy/api/api.dart';
import 'package:harpy/components/components.dart';
import 'package:harpy/core/core.dart';

/// Handles a tweet list response in an isolate.
///
/// Only the parent [LegacyTweetData] of a reply chain will be in the returned
/// list and will include all of its replies.
Future<BuiltList<LegacyTweetData>> handleTweets(
  List<Tweet> tweets, [
  TimelineFilter? filter,
]) {
  return compute<List<dynamic>, BuiltList<LegacyTweetData>>(
    _isolateHandleTweets,
    <dynamic>[tweets, filter],
  );
}

BuiltList<LegacyTweetData> _isolateHandleTweets(List<dynamic> arguments) {
  final tweets = arguments[0] as List<Tweet>;
  final filter = arguments[1] as TimelineFilter?;

  final tweetDataList = tweets
      .where((tweet) => !_filterTweet(tweet, filter))
      .map(LegacyTweetData.fromTweet)
      .toList();

  for (var i = 0; i < tweetDataList.length; i++) {
    final tweet = tweetDataList[i];

    if (tweet.parentTweetId != null && i != tweetDataList.length - 1) {
      // look for parent in older tweets
      for (var j = i + 1; j < tweetDataList.length; j++) {
        final olderTweet = tweetDataList[j];

        if (olderTweet.id == tweet.parentTweetId) {
          // found parent tweet, remove child tweet from list and add it to
          //   the replies of the parent tweet
          tweetDataList[j] = olderTweet.copyWith(
            replies: [...olderTweet.replies, tweet],
          );

          tweetDataList.removeAt(i);
          i--;

          break;
        }
      }
    }
  }

  // sort to make sure the tweets are in chronological order with the newest
  //  reply to a tweet pushing the parent tweet to the front
  tweetDataList.sort((a, b) {
    final targetA = a.replies.isNotEmpty ? a.replies.first : a;
    final targetB = b.replies.isNotEmpty ? b.replies.first : b;
    final originalIdA = int.tryParse(targetA.originalId) ?? 0;
    final originalIdB = int.tryParse(targetB.originalId) ?? 0;

    return originalIdB.compareTo(originalIdA);
  });

  return tweetDataList.toBuiltList();
}

bool _filterTweet(Tweet tweet, TimelineFilter? filter) {
  if (filter == null) {
    return false;
  } else {
    if (filter.excludes.retweets && tweet.retweetedStatus != null) {
      // filter retweets
      return true;
    }

    if (filter.includes.image || filter.includes.gif || filter.includes.video) {
      // filter non-media tweets
      if (tweet.extendedEntities?.media == null ||
          tweet.extendedEntities!.media!.isEmpty) {
        return true;
      }

      final hasImage = tweet.extendedEntities!.media!.first.type == kMediaPhoto;
      final hasGif = tweet.extendedEntities!.media!.first.type == kMediaGif;
      final hasVideo = tweet.extendedEntities!.media!.first.type == kMediaVideo;

      if (!(filter.includes.image && hasImage ||
          filter.includes.gif && hasGif ||
          filter.includes.video && hasVideo)) {
        return true;
      }
    }

    final tweetHashtags = tweet.entities?.hashtags
            ?.map((hashtag) => hashtag.text?.toLowerCase() ?? '')
            .toList() ??
        [];

    final tweetMentions = tweet.entities?.userMentions
            ?.map((mention) => mention.screenName?.toLowerCase() ?? '')
            .toList() ??
        [];

    final tweetText = tweet.fullText?.toLowerCase() ?? '';

    // includes

    // filter tweets where every included hashtags are not contained
    if (filter.includes.hashtags.isNotEmpty &&
        filter.includes.hashtags
            .map(_prepareHashtag)
            .every(tweetHashtags.containsNot)) {
      return true;
    }

    // filter tweets where every included mentions are not contained
    if (filter.includes.mentions.isNotEmpty &&
        filter.includes.mentions
            .map(_prepareMention)
            .every(tweetMentions.containsNot)) {
      return true;
    }

    // filter tweets where every included keywords / phrases is not contained
    if (filter.includes.phrases.isNotEmpty &&
        filter.includes.phrases
            .map((phrase) => phrase.toLowerCase())
            .every((phrase) => !tweetText.contains(phrase))) {
      return true;
    }

    // excludes

    // filter tweets where any excluded hashtags are contained
    if (filter.excludes.hashtags.isNotEmpty &&
        filter.excludes.hashtags
            .map(_prepareHashtag)
            .any(tweetHashtags.contains)) {
      return true;
    }

    // filter tweets where any excluded mentions are contained
    if (filter.excludes.mentions.isNotEmpty &&
        filter.excludes.mentions
            .map(_prepareMention)
            .any(tweetMentions.contains)) {
      return true;
    }

    // filter tweets where any excluded keywords / phrases are contained
    if (filter.excludes.phrases.isNotEmpty &&
        filter.excludes.phrases
            .map((phrase) => phrase.toLowerCase())
            .any(tweetText.contains)) {
      return true;
    }

    return false;
  }
}

String? _prepareHashtag(String hashtag) {
  return removePrependedSymbol(
    hashtag.toLowerCase(),
    const {'#', '＃'},
  );
}

String? _prepareMention(String mention) {
  return removePrependedSymbol(
    mention.toLowerCase(),
    const {'@'},
  );
}

extension on Iterable<dynamic> {
  bool containsNot(Object? element) {
    return !contains(element);
  }
}
